
const uniHttp = uni.$u

/**
 * http 请求
 */
class Http {
  headers
  /**
   * 登录
   * @returns {Promise<any>}
   */
  setLogin () {
    return new Promise((resolve, reject) => {
      uni.login({
        provider: 'weixin', // 使用微信登录
        /**
         * 接口调用成功的回调函数
         */
        success: (result) => {
          console.log('result', result)
          resolve(result.code)
        },
        /**
         * 接口调用失败的回调函数
         */
        fail: (result) => {
          console.log('fail result', result)
          reject(result)
        },
        /**
         * 接口调用结束的回调函数（调用成功、失败都会执行）
         * @param result
         */
        complete: (result) => {}
      })
    })
  }

  /**
   * 获取 getSystemInfo
   * @returns {Promise<any>}
   */
  getSystemInfo () {
    return new Promise((resolve, reject) => {
      uni.getSystemInfo({
        /**
         * 接口调用成功的回调函数
         */
        success: (result) => {
          console.log('result', result)
          resolve(result)
        },
        /**
         * 接口调用失败的回调函数
         */
        fail: (result) => {
          console.log('fail result', result)
          reject(result)
        },
        /**
         * 接口调用结束的回调函数（调用成功、失败都会执行）
         * @param result
         */
        complete: (result) => {}
      })
    })
  }

  /**
   * 请求前拦截器 获取config
   * @param config
   * @returns {Promise<{hospitalNo: *}>}
   */
  async setRequest (config = {}) {
    // 是否需要此配置 如果没有直接删除
    Reflect.deleteProperty(config, 'noHospitalNo')
    return {
      ...this.headers,
      ...config
    }
  }

  /**
   * 请求参数拦截
   * @param params
   * @param config
   * @returns {Promise<{patientId: *}>}
   */
  async setParams ({ params, config = {} }) {
    // 请求中不携带 noPatientId， 是否需要此配置 如果没有直接删除
    Reflect.deleteProperty(config, 'noPatientId')
    return {
      ...params
    }
  }

  /**
   * 响应拦截器
   * @param res
   * @returns {*}
   */
  async setResponse (res) {
    // 网络错误，直接提示并返回
    if (res.statusCode !== 200) {
      uni.$u.toast(`网络错误，错误码：${res.statusCode}`)
      return false
    }
    if (res.data.status === '0') {
      return res.data
    } else if (res.data.status === 201) {
      // 假设201为token失效，这里跳转登录
      uni.$u.toast('验证失败，请重新登录')
      //   setTimeout(() => {
      //     uni.$u.route('/pages/user/login')
      //   }, 1000)
      return false
    } else {
      // 如果返回false，则会调用Promise的reject回调，
      uni.$u.toast(`请求错误，${res.data.message}`)
      return Promise.reject(res.data)
    }
  }

  /**
   * get 方法
   * @param key
   * @param params
   * @param config
   * @returns {Promise<*|ActiveX.IXMLDOMNode|string|IDBRequest<any | undefined>|File|Function|Credential|null>}
   */
  async get (key, params = {}, config = {}) {
    // 获取 config
    const requestConfig = await this.setRequest(config)
    // 获取 params
    const requestParams = await this.setParams({ params, config })
    // 进行 HTTP 请求获取参数
    const res = await uniHttp.get(key, requestParams, requestConfig)
    // 对 res 进行包装进行处理
    return this.setResponse(res)
  }

  /**
   * post 方法
   * @param key
   * @param params
   * @param config
   * @returns {Promise<*>}
   */
  async post (key, params = {}, config = {}) {
    // 获取 config
    const requestConfig = await this.setRequest(config)
    // 获取 params
    const requestParams = await this.setParams({ params, config })
    // 进行 HTTP 请求获取参数
    const res = await uniHttp.post(key, requestParams, requestConfig)
    // 对 res 进行包装进行处理
    return this.setResponse(res)
  }
}

export default new Http()
